@@ -19,8 +19,10 @@
  */
 
 #include "config.h"
+#include "kateglobal.h"
 
 #include "katetextbuffer.h"
+#include "katesecuretextbuffer_p.h"
 #include "katetextloader.h"
 
 // this is unfortunate, but needed for performance
@@ -30,13 +32,13 @@
 
 #ifndef Q_OS_WIN
 #include <unistd.h>
-
-// needed for umask application
-#include <sys/types.h>
-#include <sys/stat.h>
 #endif
 
-#include <QSaveFile>
+#include <QFile>
+#include <QTemporaryFile>
+#include <QFileInfo>
+#include <QCryptographicHash>
+#include <QBuffer>
 
 #if 0
 #define BUFFER_DEBUG qCDebug(LOG_KTE)
@@ -47,7 +49,7 @@
 namespace Kate
 {
 
-TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize)
+TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize, bool alwaysUseKAuth)
     : QObject(parent)
     , m_document(parent)
     , m_history(*this)
@@ -61,12 +63,13 @@ TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize)
     , m_editingMinimalLineChanged(-1)
     , m_editingMaximalLineChanged(-1)
     , m_encodingProberType(KEncodingProber::Universal)
-    , m_fallbackTextCodec(0)
-    , m_textCodec(0)
+    , m_fallbackTextCodec(nullptr)
+    , m_textCodec(nullptr)
     , m_generateByteOrderMark(false)
     , m_endOfLineMode(eolUnix)
     , m_newLineAtEof(false)
     , m_lineLengthLimit(4096)
+    , m_alwaysUseKAuthForSave(alwaysUseKAuth)
 {
     // minimal block size must be > 0
     Q_ASSERT(m_blockSize > 0);
@@ -78,7 +81,7 @@ TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize)
 TextBuffer::~TextBuffer()
 {
     // remove document pointer, this will avoid any notifyAboutRangeChange to have a effect
-    m_document = 0;
+    m_document = nullptr;
 
     // not allowed during editing
     Q_ASSERT(m_editingTransactions == 0);
@@ -147,7 +150,7 @@ void TextBuffer::clear()
     m_generateByteOrderMark = false;
 
     // reset the filter device
-    m_mimeTypeForFilterDev = QLatin1String("text/plain");
+    m_mimeTypeForFilterDev = QStringLiteral("text/plain");
 
     // clear edit history
     m_history.clear();
@@ -297,7 +300,7 @@ void TextBuffer::unwrapLine(int line)
      * the previous one could even end up with zero lines
      * this call will trigger fixStartLines
      */
-    m_blocks.at(blockIndex)->unwrapLine(line, (blockIndex > 0) ? m_blocks.at(blockIndex - 1) : 0, firstLineInBlock ? (blockIndex - 1) : blockIndex);
+    m_blocks.at(blockIndex)->unwrapLine(line, (blockIndex > 0) ? m_blocks.at(blockIndex - 1) : nullptr, firstLineInBlock ? (blockIndex - 1) : blockIndex);
     --m_lines;
 
     // decrement index for later fixup, if we modified the block in front of the found one
@@ -572,7 +575,7 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
     /**
      * triple play, maximal three loading rounds
      * 0) use the given encoding, be done, if no encoding errors happen
-     * 1) use BOM to decided if unicode or if that fails, use encoding prober, if no encoding errors happen, be done
+     * 1) use BOM to decided if Unicode or if that fails, use encoding prober, if no encoding errors happen, be done
      * 2) use fallback encoding, be done, if no encoding errors happen
      * 3) use again given encoding, be done in any case
      */
@@ -601,7 +604,7 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
          */
         QTextCodec *codec = m_textCodec;
         if (i == 1) {
-            codec = 0;
+            codec = nullptr;
         } else if (i == 2) {
             codec = m_fallbackTextCodec;
         }
@@ -627,7 +630,7 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
                 break;
             }
 
-            // get unicode data for this line
+            // get Unicode data for this line
             const QChar *unicodeData = file.unicode() + offset;
 
 
@@ -760,37 +763,54 @@ void TextBuffer::setTextCodec(QTextCodec *codec)
 
 bool TextBuffer::save(const QString &filename)
 {
-    // codec must be set!
+    /**
+     * codec must be set, else below we fail!
+     */
     Q_ASSERT(m_textCodec);
 
-#ifndef Q_OS_WIN
-    const bool newFile = !QFile::exists(filename);
-#endif
+    /**
+     * construct correct filter device
+     * we try to use the same compression as for opening
+     */
+    const KCompressionDevice::CompressionType type = KFilterDev::compressionTypeForMimeType(m_mimeTypeForFilterDev);
+    QScopedPointer<KCompressionDevice> saveFile(new KCompressionDevice(filename, type));
 
     /**
-     * use QSaveFile for save write + rename
+     * if we want to use kauth anyways or we can open the file write only => use temporary file and try to use authhelper
      */
-    QSaveFile saveFile(filename);
-    saveFile.setDirectWriteFallback(true);
+    uint ownerId = -2;
+    uint groupId = -2;
+    QScopedPointer<QIODevice> temporaryBuffer;
+    if (m_alwaysUseKAuthForSave || !saveFile->open(QIODevice::WriteOnly)) {
+        /**
+         * Memorize owner and group.
+         */
+        const QFileInfo fileInfo(filename);
+        if (fileInfo.exists()) {
+            ownerId = fileInfo.ownerId();
+            groupId = fileInfo.groupId();
+        }
 
-    if (!saveFile.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-        return false;
-    }
+        // if that fails we need more privileges to save this file
+        // -> we write to a temporary file and then send its path to KAuth action for privileged save
+        temporaryBuffer.reset(new QBuffer());
 
-    /**
-     * construct correct filter device and try to open
-     */
-    KCompressionDevice::CompressionType type = KFilterDev::compressionTypeForMimeType(m_mimeTypeForFilterDev);
-    KCompressionDevice file(&saveFile, false, type);
+        // open buffer for write and read (read is used for checksum computing and writing to temporary file)
+        if (!temporaryBuffer->open(QIODevice::ReadWrite)) {
+            return false;
+        }
 
-    if (!file.open(QIODevice::WriteOnly)) {
-        return false;
+        // we are now saving to a temporary buffer with potential compression proxy
+        saveFile.reset(new KCompressionDevice(temporaryBuffer.data(), false, type));
+        if (!saveFile->open(QIODevice::WriteOnly)) {
+            return false;
+        }
     }
 
     /**
      * construct stream + disable Unicode headers
      */
-    QTextStream stream(&file);
+    QTextStream stream(saveFile.data());
     stream.setCodec(QTextCodec::codecForName("UTF-16"));
 
     // set the correct codec
@@ -800,26 +820,30 @@ bool TextBuffer::save(const QString &filename)
     stream.setGenerateByteOrderMark(generateByteOrderMark());
 
     // our loved eol string ;)
-    QString eol = QLatin1String("\n"); //m_doc->config()->eolString ();
+    QString eol = QStringLiteral("\n");
     if (endOfLineMode() == eolDos) {
-        eol = QLatin1String("\r\n");
+        eol = QStringLiteral("\r\n");
     } else if (endOfLineMode() == eolMac) {
-        eol = QLatin1String("\r");
+        eol = QStringLiteral("\r");
     }
 
     // just dump the lines out ;)
     for (int i = 0; i < m_lines; ++i) {
-        // get line to save
-        Kate::TextLine textline = line(i);
-
-        stream << textline->text();
+        // dump current line
+        stream << line(i)->text();
 
         // append correct end of line string
         if ((i + 1) < m_lines) {
             stream << eol;
         }
+
+        // early out on stream errors
+        if (stream.status() != QTextStream::Ok) {
+            return false;
+        }
     }
 
+    // do we need to add a trailing newline char?
     if (m_newLineAtEof) {
         Q_ASSERT(m_lines > 0); // see .h file
         const Kate::TextLine lastLine = line(m_lines - 1);
@@ -832,56 +856,84 @@ bool TextBuffer::save(const QString &filename)
     // flush stream
     stream.flush();
 
-    // close and delete file
-    file.close();
-
-    // flush file
-    if (!saveFile.flush()) {
+    // only finalize if stream status == OK
+    if (stream.status() != QTextStream::Ok) {
         return false;
     }
 
-#ifndef Q_OS_WIN
-    // ensure that the file is written to disk
-#ifdef HAVE_FDATASYNC
-    fdatasync(saveFile.handle());
-#else
-    fsync(saveFile.handle());
-#endif
-#endif
+    // close the file, we might want to read from underlying buffer below
+    saveFile->close();
 
     // did save work?
-    // only finalize if stream status == OK
-    bool ok = (stream.status() == QTextStream::Ok) && saveFile.commit();
-
-    // remember this revision as last saved if we had success!
-    if (ok) {
-        m_history.setLastSavedRevision();
+    if (saveFile->error() != QFileDevice::NoError) {
+        BUFFER_DEBUG << "Saving file " << filename << "failed with error" << saveFile->errorString();
+        return false;
     }
 
-#ifndef Q_OS_WIN
-    if (ok && newFile) { // QTemporaryFile sets permissions to 0600, so fixing this
-        const mode_t mask = umask(0);
-        umask(mask);
+    /**
+     * if we used a QBuffer for kauth, try to move that to the real location with the helper
+     */
+    if (temporaryBuffer) {
+        // temporary buffer was used to save the file
+        // -> computing checksum
+        // -> saving to temporary file
+        // -> copying the temporary file to the original file location with KAuth action
+        QTemporaryFile tempFile;
+        if (!tempFile.open()) {
+            return false;
+        }
 
-        const mode_t fileMode = 0666 & ~mask;
-        chmod(QFile::encodeName(filename).constData(), fileMode);
-    }
-#endif
+        // go to QBuffer start
+        temporaryBuffer->seek(0);
 
-    // report CODEC + ERRORS
-    BUFFER_DEBUG << "Saved file " << filename << "with codec" << m_textCodec->name() << (ok ? "without" : "with") << "errors";
+        // read contents of QBuffer and add them to checksum utility as well as to QTemporaryFile
+        char buffer[bufferLength];
+        qint64 read = -1;
+        QCryptographicHash cryptographicHash(SecureTextBuffer::checksumAlgorithm);
+        while ((read = temporaryBuffer->read(buffer, bufferLength)) > 0) {
+            cryptographicHash.addData(buffer, read);
+            if (tempFile.write(buffer, read) == -1) {
+                return false;
+            }
+        }
+        if (!tempFile.flush()) {
+            return false;
+        }
 
-    if (ok) {
-        markModifiedLinesAsSaved();
+        // prepare data for KAuth action
+        QVariantMap kAuthActionArgs;
+        kAuthActionArgs.insert(QLatin1String("sourceFile"), tempFile.fileName());
+        kAuthActionArgs.insert(QLatin1String("targetFile"), filename);
+        kAuthActionArgs.insert(QLatin1String("checksum"), cryptographicHash.result());
+        kAuthActionArgs.insert(QLatin1String("ownerId"), ownerId);
+        kAuthActionArgs.insert(QLatin1String("groupId"), groupId);
+
+        // call save with elevated privileges
+        if (KTextEditor::EditorPrivate::unitTestMode()) {
+            // unit testing purposes only
+            if (!SecureTextBuffer::savefile(kAuthActionArgs).succeeded()) {
+                return false;
+            }
+        } else {
+            KAuth::Action kAuthSaveAction(QLatin1String("org.kde.ktexteditor.katetextbuffer.savefile"));
+            kAuthSaveAction.setHelperId(QLatin1String("org.kde.ktexteditor.katetextbuffer"));
+            kAuthSaveAction.setArguments(kAuthActionArgs);
+            KAuth::ExecuteJob *job = kAuthSaveAction.execute();
+            if (!job->exec()) {
+                return false;
+            }
+        }
     }
 
-    // emit signal on success
-    if (ok) {
-        emit saved(filename);
-    }
+    // remember this revision as last saved
+    m_history.setLastSavedRevision();
 
-    // return success or not
-    return ok;
+    // inform that we have saved the state
+    markModifiedLinesAsSaved();
+
+    // emit that file was saved and be done
+    emit saved(filename);
+    return true;
 }
 
 void TextBuffer::notifyAboutRangeChange(KTextEditor::View *view, int startLine, int endLine, bool rangeWithAttribute)
