@@ -45,7 +45,7 @@ using namespace KTextEditor;
 class HierarchicalModelHandler
 {
 public:
-    HierarchicalModelHandler(CodeCompletionModel *model);
+    explicit HierarchicalModelHandler(CodeCompletionModel *model);
     void addValue(CodeCompletionModel::ExtraItemDataRoles role, const QVariant &value);
     //Walks the index upwards and collects all defined completion-roles on the way
     void collectRoles(const QModelIndex &index);
@@ -142,26 +142,11 @@ void HierarchicalModelHandler::addValue(CodeCompletionModel::ExtraItemDataRoles
 
 KateCompletionModel::KateCompletionModel(KateCompletionWidget *parent)
     : ExpandingWidgetModel(parent)
-    , m_hasGroups(false)
-    , m_matchCaseSensitivity(Qt::CaseInsensitive)
     , m_ungrouped(new Group({}, 0, this))
     , m_argumentHints(new Group(i18n("Argument-hints"), -1, this))
     , m_bestMatches(new Group(i18n("Best matches"), BestMatchesProperty, this))
-    , m_sortingEnabled(false)
-    , m_sortingAlphabetical(false)
-    , m_isSortingByInheritance(false)
-    , m_sortingCaseSensitivity(Qt::CaseInsensitive)
-    , m_filteringEnabled(false)
-    , m_filterContextMatchesOnly(false)
-    , m_filterByAttribute(false)
     , m_filterAttributes(KTextEditor::CodeCompletionModel::NoProperty)
-    , m_maximumInheritanceDepth(0)
-    , m_groupingEnabled(false)
-    , m_accessConst(false)
-    , m_accessStatic(false)
-    , m_accesSignalSlot(false)
-    , m_columnMergingEnabled(false)
-//   , m_haveExactMatch(false)
+
 {
     m_emptyGroups.append(m_ungrouped);
     m_emptyGroups.append(m_argumentHints);
@@ -208,7 +193,7 @@ QVariant KateCompletionModel::data(const QModelIndex &index, int role) const
     //groupOfParent returns a group when the index is a member of that group, but not the group head/label.
     if (!hasGroups() || groupOfParent(index)) {
         if ( role == Qt::TextAlignmentRole ) {
-            if (isColumnMergingEnabled() && m_columnMerges.count()) {
+            if (isColumnMergingEnabled() && !m_columnMerges.isEmpty()) {
                 int c = 0;
                 foreach (const QList<int> &list, m_columnMerges) {
                     if (index.column() < c + list.size()) {
@@ -229,7 +214,7 @@ QVariant KateCompletionModel::data(const QModelIndex &index, int role) const
         }
 
         // Merge text for column merging
-        if (role == Qt::DisplayRole && m_columnMerges.count() && isColumnMergingEnabled()) {
+        if (role == Qt::DisplayRole && !m_columnMerges.isEmpty() && isColumnMergingEnabled()) {
             QString text;
             foreach (int column, m_columnMerges[index.column()]) {
                 QModelIndex sourceIndex = mapToSource(createIndex(index.row(), column, index.internalPointer()));
@@ -365,7 +350,7 @@ int KateCompletionModel::contextMatchQuality(const ModelRow &source) const
 Qt::ItemFlags KateCompletionModel::flags(const QModelIndex &index) const
 {
     if (!hasCompletionModel() || !index.isValid()) {
-        return 0;
+        return Qt::NoItemFlags;
     }
 
     if (!hasGroups() || groupOfParent(index)) {
@@ -747,11 +732,11 @@ KateCompletionModel::Group *KateCompletionModel::fetchGroup(int attribute, const
 
     if (groupingMethod() & ScopeType) {
         if (attribute & KTextEditor::CodeCompletionModel::GlobalScope) {
-            st = QLatin1String("Global");
+            st = QStringLiteral("Global");
         } else if (attribute & KTextEditor::CodeCompletionModel::NamespaceScope) {
-            st = QLatin1String("Namespace");
+            st = QStringLiteral("Namespace");
         } else if (attribute & KTextEditor::CodeCompletionModel::LocalScope) {
-            st = QLatin1String("Local");
+            st = QStringLiteral("Local");
         }
 
         title = st;
@@ -767,11 +752,11 @@ KateCompletionModel::Group *KateCompletionModel::fetchGroup(int attribute, const
 
     if (groupingMethod() & AccessType) {
         if (attribute & KTextEditor::CodeCompletionModel::Public) {
-            at = QLatin1String("Public");
+            at = QStringLiteral("Public");
         } else if (attribute & KTextEditor::CodeCompletionModel::Protected) {
-            at = QLatin1String("Protected");
+            at = QStringLiteral("Protected");
         } else if (attribute & KTextEditor::CodeCompletionModel::Private) {
-            at = QLatin1String("Private");
+            at = QStringLiteral("Private");
         }
 
         if (accessIncludeStatic() && attribute & KTextEditor::CodeCompletionModel::Static) {
@@ -843,12 +828,12 @@ KateCompletionModel::Group *KateCompletionModel::groupForIndex(const QModelIndex
         if (!hasGroups()) {
             return m_ungrouped;
         } else {
-            return 0L;
+            return nullptr;
         }
     }
 
     if (groupOfParent(index)) {
-        return 0L;
+        return nullptr;
     }
 
     if (index.row() < 0 || index.row() >= m_rowTable.count()) {
@@ -1857,7 +1842,7 @@ bool KateCompletionModel::shouldMatchHideCompletionList() const
     // @todo Make this faster
 
     bool doHide = false;
-    CodeCompletionModel *hideModel = 0;
+    CodeCompletionModel *hideModel = nullptr;
 
     foreach (Group *group, m_rowTable)
         foreach (const Item &item, group->filtered)
@@ -1987,7 +1972,7 @@ static inline bool containsAtWordBeginning(const QString &word, const QString &t
         }
     }
     return false;
-};
+}
 
 KateCompletionModel::Item::MatchType KateCompletionModel::Item::match()
 {
@@ -2006,7 +1991,7 @@ KateCompletionModel::Item::MatchType KateCompletionModel::Item::match()
     matchCompletion = (m_nameColumn.startsWith(match, model->matchCaseSensitivity()) ? StartsWithMatch : NoMatch);
     if (matchCompletion == NoMatch) {
         // if no match, try for "contains"
-        // Only match when the occurence is at a "word" beginning, marked by
+        // Only match when the occurrence is at a "word" beginning, marked by
         // an underscore or a capital. So Foo matches BarFoo and Bar_Foo, but not barfoo.
         // Starting at 1 saves looking at the beginning of the word, that was already checked above.
         if (containsAtWordBeginning(m_nameColumn, match, model->matchCaseSensitivity())) {
@@ -2315,8 +2300,8 @@ void KateCompletionModel::updateBestMatches()
             }
 
             {
-                uint size = m_ungrouped->filtered.size();
-                for (uint a = 0; a < size; ++a)
+                int size = m_ungrouped->filtered.size();
+                for (int a = 0; a < size; ++a)
                     if (!movedToFront.contains(a)) {
                         newFiltered.append(m_ungrouped->filtered[a]);
                     }
