@@ -36,6 +36,7 @@
 #include <ktexteditor/movinginterface.h>
 #include <ktexteditor/message.h>
 #include <ktexteditor/mainwindow.h>
+#include <ktexteditor/inlinenoteinterface.h>
 
 #include <ktexteditor_export.h>
 #include "katetextline.h"
@@ -97,12 +98,12 @@ class KTEXTEDITOR_EXPORT KTextEditor::DocumentPrivate : public KTextEditor::Docu
 public:
     explicit DocumentPrivate(bool bSingleViewMode = false, bool bReadOnly = false,
                           QWidget *parentWidget = nullptr, QObject * = nullptr);
-    ~DocumentPrivate();
+    ~DocumentPrivate() override;
 
     using ReadWritePart::closeUrl;
-    bool closeUrl() Q_DECL_OVERRIDE;
+    bool closeUrl() override;
 
-    bool openUrl(const QUrl &url) Q_DECL_OVERRIDE;
+    bool openUrl(const QUrl &url) override;
 
     KTextEditor::Range rangeOnLine(KTextEditor::Range range, int line) const;
 
@@ -115,7 +116,7 @@ public:
     /**
      * @return The widget defined by this part, set by setWidget().
      */
-    QWidget *widget() Q_DECL_OVERRIDE;
+    QWidget *widget() override;
 
 public:
     bool readOnly() const
@@ -136,11 +137,11 @@ private:
     // KTextEditor::Document stuff
     //
 public:
-    KTextEditor::View *createView(QWidget *parent, KTextEditor::MainWindow *mainWindow = nullptr) Q_DECL_OVERRIDE;
+    KTextEditor::View *createView(QWidget *parent, KTextEditor::MainWindow *mainWindow = nullptr) override;
 
-    QList<KTextEditor::View *> views() const Q_DECL_OVERRIDE
+    QList<KTextEditor::View *> views() const override
     {
-        return m_views.keys();
+        return m_viewsCache;
     }
 
     virtual KTextEditor::View *activeView() const
@@ -150,50 +151,50 @@ public:
 
 private:
     QHash<KTextEditor::View *, KTextEditor::ViewPrivate *> m_views;
-    KTextEditor::View *m_activeView;
+    KTextEditor::View *m_activeView = nullptr;
 
     //
     // KTextEditor::EditInterface stuff
     //
 public Q_SLOTS:
-    bool setText(const QString &) Q_DECL_OVERRIDE;
-    bool setText(const QStringList &text) Q_DECL_OVERRIDE;
-    bool clear() Q_DECL_OVERRIDE;
+    bool setText(const QString &) override;
+    bool setText(const QStringList &text) override;
+    bool clear() override;
 
-    bool insertText(const KTextEditor::Cursor &position, const QString &s, bool block = false) Q_DECL_OVERRIDE;
-    bool insertText(const KTextEditor::Cursor &position, const QStringList &text, bool block = false) Q_DECL_OVERRIDE;
+    bool insertText(const KTextEditor::Cursor &position, const QString &s, bool block = false) override;
+    bool insertText(const KTextEditor::Cursor &position, const QStringList &text, bool block = false) override;
 
-    bool insertLine(int line, const QString &s) Q_DECL_OVERRIDE;
-    bool insertLines(int line, const QStringList &s) Q_DECL_OVERRIDE;
+    bool insertLine(int line, const QString &s) override;
+    bool insertLines(int line, const QStringList &s) override;
 
-    bool removeText(const KTextEditor::Range &range, bool block = false) Q_DECL_OVERRIDE;
-    bool removeLine(int line) Q_DECL_OVERRIDE;
+    bool removeText(const KTextEditor::Range &range, bool block = false) override;
+    bool removeLine(int line) override;
 
-    bool replaceText(const KTextEditor::Range &range, const QString &s, bool block = false) Q_DECL_OVERRIDE;
+    bool replaceText(const KTextEditor::Range &range, const QString &s, bool block = false) override;
 
     // unhide method...
-    bool replaceText(const KTextEditor::Range &r, const QStringList &l, bool b) Q_DECL_OVERRIDE
+    bool replaceText(const KTextEditor::Range &r, const QStringList &l, bool b) override
     {
         return KTextEditor::Document::replaceText(r, l, b);
     }
 
 public:
-    bool isEditingTransactionRunning() const Q_DECL_OVERRIDE;
-    QString text(const KTextEditor::Range &range, bool blockwise = false) const Q_DECL_OVERRIDE;
-    QStringList textLines(const KTextEditor::Range &range, bool block = false) const Q_DECL_OVERRIDE;
-    QString text() const Q_DECL_OVERRIDE;
-    QString line(int line) const Q_DECL_OVERRIDE;
-    QChar characterAt(const KTextEditor::Cursor &position) const Q_DECL_OVERRIDE;
-    QString wordAt(const KTextEditor::Cursor &cursor) const Q_DECL_OVERRIDE;
-    KTextEditor::Range wordRangeAt(const KTextEditor::Cursor &cursor) const Q_DECL_OVERRIDE;
-    bool isValidTextPosition(const KTextEditor::Cursor& cursor) const Q_DECL_OVERRIDE;
-    int lines() const Q_DECL_OVERRIDE;
-    bool isLineModified(int line) const Q_DECL_OVERRIDE;
-    bool isLineSaved(int line) const Q_DECL_OVERRIDE;
-    bool isLineTouched(int line) const Q_DECL_OVERRIDE;
-    KTextEditor::Cursor documentEnd() const Q_DECL_OVERRIDE;
-    int totalCharacters() const Q_DECL_OVERRIDE;
-    int lineLength(int line) const Q_DECL_OVERRIDE;
+    bool isEditingTransactionRunning() const override;
+    QString text(const KTextEditor::Range &range, bool blockwise = false) const override;
+    QStringList textLines(const KTextEditor::Range &range, bool block = false) const override;
+    QString text() const override;
+    QString line(int line) const override;
+    QChar characterAt(const KTextEditor::Cursor &position) const override;
+    QString wordAt(const KTextEditor::Cursor &cursor) const override;
+    KTextEditor::Range wordRangeAt(const KTextEditor::Cursor &cursor) const override;
+    bool isValidTextPosition(const KTextEditor::Cursor& cursor) const override;
+    int lines() const override;
+    bool isLineModified(int line) const override;
+    bool isLineSaved(int line) const override;
+    bool isLineTouched(int line) const override;
+    KTextEditor::Cursor documentEnd() const override;
+    int totalCharacters() const override;
+    int lineLength(int line) const override;
 
 Q_SIGNALS:
     void charactersSemiInteractivelyInserted(const KTextEditor::Cursor &position, const QString &text);
@@ -355,10 +356,10 @@ public:
     KTextEditor::Cursor lastEditingPosition(EditingPositionKind nextOrPrevious, KTextEditor::Cursor);
 
 private:
-    int editSessionNumber;
+    int editSessionNumber = 0;
     QStack<int> editStateStack;
-    bool editIsRunning;
-    bool m_undoMergeAllEdits;
+    bool editIsRunning = false;
+    bool m_undoMergeAllEdits = false;
     QStack<QSharedPointer<KTextEditor::MovingCursor>> m_editingStack;
     int m_editingStackPosition = -1;
     static const int s_editingStackSizeLimit = 32;
@@ -416,52 +417,52 @@ public:
     /**
      * @copydoc KTextEditor::Document::defaultStyleAt()
      */
-    KTextEditor::DefaultStyle defaultStyleAt(const KTextEditor::Cursor &position) const Q_DECL_OVERRIDE;
+    KTextEditor::DefaultStyle defaultStyleAt(const KTextEditor::Cursor &position) const override;
 
     /**
      * Return the name of the currently used mode
      * \return name of the used mode
      */
-    QString mode() const Q_DECL_OVERRIDE;
+    QString mode() const override;
 
     /**
      * Return the name of the currently used mode
      * \return name of the used mode
      */
-    QString highlightingMode() const Q_DECL_OVERRIDE;
+    QString highlightingMode() const override;
 
     /**
      * Return a list of the names of all possible modes
      * \return list of mode names
      */
-    QStringList modes() const Q_DECL_OVERRIDE;
+    QStringList modes() const override;
 
     /**
      * Return a list of the names of all possible modes
      * \return list of mode names
      */
-    QStringList highlightingModes() const Q_DECL_OVERRIDE;
+    QStringList highlightingModes() const override;
 
     /**
      * Set the current mode of the document by giving its name
      * \param name name of the mode to use for this document
      * \return \e true on success, otherwise \e false
      */
-    bool setMode(const QString &name) Q_DECL_OVERRIDE;
+    bool setMode(const QString &name) override;
 
     /**
      * Set the current mode of the document by giving its name
      * \param name name of the mode to use for this document
      * \return \e true on success, otherwise \e false
      */
-    bool setHighlightingMode(const QString &name) Q_DECL_OVERRIDE;
+    bool setHighlightingMode(const QString &name) override;
     /**
      * Returns the name of the section for a highlight given its @p index in the highlight
      * list (as returned by highlightModes()).
      * You can use this function to build a tree of the highlight names, organized in sections.
      * \param index in the highlight list for which to find the section name.
      */
-    QString highlightingModeSection(int index) const Q_DECL_OVERRIDE;
+    QString highlightingModeSection(int index) const override;
 
     /**
      * Returns the name of the section for a mode given its @p index in the highlight
@@ -469,7 +470,7 @@ public:
      * You can use this function to build a tree of the mode names, organized in sections.
      * \param index index in the highlight list for which to find the section name.
      */
-    QString modeSection(int index) const Q_DECL_OVERRIDE;
+    QString modeSection(int index) const override;
 
     /*
      * Helpers....
@@ -502,7 +503,7 @@ public:
      * \param flags additional flags
      * \see writeSessionConfig()
      */
-    void readSessionConfig(const KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) Q_DECL_OVERRIDE;
+    void readSessionConfig(const KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) override;
 
     /**
      * Write session settings to the \p config.
@@ -512,7 +513,7 @@ public:
      * \param flags additional flags
      * \see readSessionConfig()
      */
-    void writeSessionConfig(KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) Q_DECL_OVERRIDE;
+    void writeSessionConfig(KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) override;
 
 Q_SIGNALS:
     void configChanged();
@@ -521,13 +522,13 @@ Q_SIGNALS:
     // KTextEditor::MarkInterface
     //
 public Q_SLOTS:
-    void setMark(int line, uint markType) Q_DECL_OVERRIDE;
-    void clearMark(int line) Q_DECL_OVERRIDE;
+    void setMark(int line, uint markType) override;
+    void clearMark(int line) override;
 
-    void addMark(int line, uint markType) Q_DECL_OVERRIDE;
-    void removeMark(int line, uint markType) Q_DECL_OVERRIDE;
+    void addMark(int line, uint markType) override;
+    void removeMark(int line, uint markType) override;
 
-    void clearMarks() Q_DECL_OVERRIDE;
+    void clearMarks() override;
 
     void requestMarkTooltip(int line, QPoint position);
 
@@ -537,19 +538,19 @@ public Q_SLOTS:
     ///Returns true if the context-menu event should not further be processed
     bool handleMarkContextMenu(int line, QPoint position);
 
-    void setMarkPixmap(MarkInterface::MarkTypes, const QPixmap &) Q_DECL_OVERRIDE;
+    void setMarkPixmap(MarkInterface::MarkTypes, const QPixmap &) override;
 
-    void setMarkDescription(MarkInterface::MarkTypes, const QString &) Q_DECL_OVERRIDE;
+    void setMarkDescription(MarkInterface::MarkTypes, const QString &) override;
 
-    void setEditableMarks(uint markMask) Q_DECL_OVERRIDE;
+    void setEditableMarks(uint markMask) override;
 
 public:
-    uint mark(int line) Q_DECL_OVERRIDE;
-    const QHash<int, KTextEditor::Mark *> &marks() Q_DECL_OVERRIDE;
-    QPixmap markPixmap(MarkInterface::MarkTypes) const Q_DECL_OVERRIDE;
-    QString markDescription(MarkInterface::MarkTypes) const Q_DECL_OVERRIDE;
+    uint mark(int line) override;
+    const QHash<int, KTextEditor::Mark *> &marks() override;
+    QPixmap markPixmap(MarkInterface::MarkTypes) const override;
+    QString markDescription(MarkInterface::MarkTypes) const override;
     virtual QColor markColor(MarkInterface::MarkTypes) const;
-    uint editableMarks() const Q_DECL_OVERRIDE;
+    uint editableMarks() const override;
 
 Q_SIGNALS:
     void markToolTipRequested(KTextEditor::Document *document, KTextEditor::Mark mark, QPoint position, bool &handled);
@@ -558,20 +559,20 @@ Q_SIGNALS:
 
     void markClicked(KTextEditor::Document *document, KTextEditor::Mark mark, bool &handled);
 
-    void marksChanged(KTextEditor::Document *) Q_DECL_OVERRIDE;
-    void markChanged(KTextEditor::Document *, KTextEditor::Mark, KTextEditor::MarkInterface::MarkChangeAction) Q_DECL_OVERRIDE;
+    void marksChanged(KTextEditor::Document *) override;
+    void markChanged(KTextEditor::Document *, KTextEditor::Mark, KTextEditor::MarkInterface::MarkChangeAction) override;
 
 private:
     QHash<int, KTextEditor::Mark *> m_marks;
     QHash<int, QPixmap>           m_markPixmaps;
     QHash<int, QString>           m_markDescriptions;
-    uint                        m_editableMarks;
+    uint                        m_editableMarks = markType01;
 
     // KTextEditor::PrintInterface
     //
 public Q_SLOTS:
-    bool print() Q_DECL_OVERRIDE;
-    void printPreview() Q_DECL_OVERRIDE;
+    bool print() override;
+    void printPreview() override;
 
     //
     // KTextEditor::DocumentInfoInterface ( ### unfinished )
@@ -582,7 +583,7 @@ public:
      *
      * @return the name of the mimetype for the document.
      */
-    QString mimeType() Q_DECL_OVERRIDE;
+    QString mimeType() override;
 
     //
     // once was KTextEditor::VariableInterface
@@ -624,7 +625,7 @@ public:
      * @param insertBehavior insertion behavior
      * @return new moving cursor for the document
      */
-    KTextEditor::MovingCursor *newMovingCursor(const KTextEditor::Cursor &position, KTextEditor::MovingCursor::InsertBehavior insertBehavior = KTextEditor::MovingCursor::MoveOnInsert) Q_DECL_OVERRIDE;
+    KTextEditor::MovingCursor *newMovingCursor(const KTextEditor::Cursor &position, KTextEditor::MovingCursor::InsertBehavior insertBehavior = KTextEditor::MovingCursor::MoveOnInsert) override;
 
     /**
      * Create a new moving range for this document.
@@ -634,32 +635,32 @@ public:
      * @return new moving range for the document
      */
     KTextEditor::MovingRange *newMovingRange(const KTextEditor::Range &range, KTextEditor::MovingRange::InsertBehaviors insertBehaviors = KTextEditor::MovingRange::DoNotExpand
-            , KTextEditor::MovingRange::EmptyBehavior emptyBehavior = KTextEditor::MovingRange::AllowEmpty) Q_DECL_OVERRIDE;
+            , KTextEditor::MovingRange::EmptyBehavior emptyBehavior = KTextEditor::MovingRange::AllowEmpty) override;
 
     /**
      * Current revision
      * @return current revision
      */
-    qint64 revision() const Q_DECL_OVERRIDE;
+    qint64 revision() const override;
 
     /**
      * Last revision the buffer got successful saved
      * @return last revision buffer got saved, -1 if none
      */
-    qint64 lastSavedRevision() const Q_DECL_OVERRIDE;
+    qint64 lastSavedRevision() const override;
 
     /**
      * Lock a revision, this will keep it around until released again.
      * But all revisions will always be cleared on buffer clear() (and therefor load())
      * @param revision revision to lock
      */
-    void lockRevision(qint64 revision) Q_DECL_OVERRIDE;
+    void lockRevision(qint64 revision) override;
 
     /**
      * Release a revision.
      * @param revision revision to release
      */
-    void unlockRevision(qint64 revision) Q_DECL_OVERRIDE;
+    void unlockRevision(qint64 revision) override;
 
     /**
      * Transform a cursor from one revision to an other.
@@ -668,7 +669,7 @@ public:
      * @param fromRevision from this revision we want to transform
      * @param toRevision to this revision we want to transform, default of -1 is current revision
      */
-    void transformCursor(KTextEditor::Cursor &cursor, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) Q_DECL_OVERRIDE;
+    void transformCursor(KTextEditor::Cursor &cursor, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
 
     /**
      * Transform a cursor from one revision to an other.
@@ -678,7 +679,7 @@ public:
      * @param fromRevision from this revision we want to transform
      * @param toRevision to this revision we want to transform, default of -1 is current revision
      */
-    void transformCursor(int &line, int &column, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) Q_DECL_OVERRIDE;
+    void transformCursor(int &line, int &column, KTextEditor::MovingCursor::InsertBehavior insertBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
 
     /**
      * Transform a range from one revision to an other.
@@ -688,7 +689,7 @@ public:
      * @param fromRevision from this revision we want to transform
      * @param toRevision to this revision we want to transform, default of -1 is current revision
      */
-    void transformRange(KTextEditor::Range &range, KTextEditor::MovingRange::InsertBehaviors insertBehaviors, KTextEditor::MovingRange::EmptyBehavior emptyBehavior, qint64 fromRevision, qint64 toRevision = -1) Q_DECL_OVERRIDE;
+    void transformRange(KTextEditor::Range &range, KTextEditor::MovingRange::InsertBehaviors insertBehaviors, KTextEditor::MovingRange::EmptyBehavior emptyBehavior, qint64 fromRevision, qint64 toRevision = -1) override;
 
     //
     // MovingInterface Signals
@@ -712,14 +713,14 @@ Q_SIGNALS:
     //
 public:
 
-    void setAnnotationModel(KTextEditor::AnnotationModel *model) Q_DECL_OVERRIDE;
-    KTextEditor::AnnotationModel *annotationModel() const Q_DECL_OVERRIDE;
+    void setAnnotationModel(KTextEditor::AnnotationModel *model) override;
+    KTextEditor::AnnotationModel *annotationModel() const override;
 
 Q_SIGNALS:
     void annotationModelChanged(KTextEditor::AnnotationModel *, KTextEditor::AnnotationModel *);
 
 private:
-    KTextEditor::AnnotationModel *m_annotationModel;
+    KTextEditor::AnnotationModel *m_annotationModel = nullptr;
 
     //
     // KParts::ReadWrite stuff
@@ -730,18 +731,18 @@ public:
      * the framework abstracts the loading of remote files
      * @return success
      */
-    bool openFile() Q_DECL_OVERRIDE;
+    bool openFile() override;
 
     /**
      * save the file obtained by the kparts framework
      * the framework abstracts the uploading of remote files
      * @return success
      */
-    bool saveFile() Q_DECL_OVERRIDE;
+    bool saveFile() override;
 
-    void setReadWrite(bool rw = true) Q_DECL_OVERRIDE;
+    void setReadWrite(bool rw = true) override;
 
-    void setModified(bool m) Q_DECL_OVERRIDE;
+    void setModified(bool m) override;
 
 private:
     void activateDirWatch(const QString &useFileName = QString());
@@ -907,7 +908,7 @@ public:
     KTextEditor::Range findMatchingBracket(const KTextEditor::Cursor & start, int maxLines);
 
 public:
-    QString documentName() const Q_DECL_OVERRIDE
+    QString documentName() const override
     {
         return m_docName;
     }
@@ -924,9 +925,9 @@ public:
         return m_modOnHd;
     }
 
-    void setModifiedOnDisk(ModifiedOnDiskReason reason) Q_DECL_OVERRIDE;
+    void setModifiedOnDisk(ModifiedOnDiskReason reason) override;
 
-    void setModifiedOnDiskWarning(bool on) Q_DECL_OVERRIDE;
+    void setModifiedOnDiskWarning(bool on) override;
 
 public Q_SLOTS:
     /**
@@ -938,16 +939,16 @@ public Q_SLOTS:
     /**
      * Reloads the current document from disk if possible
      */
-    bool documentReload() Q_DECL_OVERRIDE;
+    bool documentReload() override;
 
-    bool documentSave() Q_DECL_OVERRIDE;
-    bool documentSaveAs() Q_DECL_OVERRIDE;
+    bool documentSave() override;
+    bool documentSaveAs() override;
     bool documentSaveAsWithEncoding(const QString &encoding);
     bool documentSaveCopyAs();
 
-    bool save() Q_DECL_OVERRIDE;
+    bool save() override;
 public:
-    bool saveAs(const QUrl &url) Q_DECL_OVERRIDE;
+    bool saveAs(const QUrl &url) override;
 
 Q_SIGNALS:
     /**
@@ -956,7 +957,7 @@ Q_SIGNALS:
      * @param isModified indicates the file was modified rather than created or deleted
      * @param reason the reason we are emitting the signal.
      */
-    void modifiedOnDisk(KTextEditor::Document *doc, bool isModified, KTextEditor::ModificationInterface::ModifiedOnDiskReason reason) Q_DECL_OVERRIDE;
+    void modifiedOnDisk(KTextEditor::Document *doc, bool isModified, KTextEditor::ModificationInterface::ModifiedOnDiskReason reason) override;
 
 private:
     // helper to handle the embedded notification for externally modified files
@@ -968,8 +969,8 @@ private Q_SLOTS:
     void onModOnHdIgnore();
 
 public:
-    bool setEncoding(const QString &e) Q_DECL_OVERRIDE;
-    QString encoding() const Q_DECL_OVERRIDE;
+    bool setEncoding(const QString &e) override;
+    QString encoding() const override;
 
 public Q_SLOTS:
     void setWordWrap(bool on);
@@ -1029,7 +1030,7 @@ public:
      * Returns a git compatible sha1 checksum of this document on disk.
      * @return checksum for this document on disk
      */
-    QByteArray checksum() const Q_DECL_OVERRIDE;
+    QByteArray checksum() const override;
 
     void updateFileType(const QString &newType, bool user = false);
 
@@ -1075,34 +1076,32 @@ private:
     // indenter
     KateAutoIndent *const m_indenter;
 
-    bool m_hlSetByUser;
-    bool m_bomSetByUser;
-    bool m_indenterSetByUser;
-    bool m_userSetEncodingForNextReload;
+    bool m_hlSetByUser = false;
+    bool m_bomSetByUser = false;
+    bool m_indenterSetByUser = false;
+    bool m_userSetEncodingForNextReload = false;
 
-    bool m_modOnHd;
-    ModifiedOnDiskReason m_modOnHdReason;
-    ModifiedOnDiskReason m_prevModOnHdReason;
+    bool m_modOnHd = false;
+    ModifiedOnDiskReason m_modOnHdReason = OnDiskUnmodified;
+    ModifiedOnDiskReason m_prevModOnHdReason = OnDiskUnmodified;
 
     QString m_docName;
-    int m_docNameNumber;
+    int m_docNameNumber = 0;
 
     // file type !!!
     QString m_fileType;
-    bool m_fileTypeSetByUser;
+    bool m_fileTypeSetByUser = false;
 
     /**
      * document is still reloading a file
      */
-    bool m_reloading;
+    bool m_reloading = false;
 
 public Q_SLOTS:
     void slotQueryClose_save(bool *handled, bool *abortClosing);
 
 public:
-    bool queryClose() Q_DECL_OVERRIDE;
-
-    void makeAttribs(bool needInvalidate = true);
+    bool queryClose() override;
 
     static bool checkOverwrite(QUrl u, QWidget *parent);
 
@@ -1122,6 +1121,8 @@ public:
     void updateConfig();
 
 private:
+    void makeAttribs(bool needInvalidate = true);
+
     KateDocumentConfig *const m_config;
 
     /**
@@ -1167,30 +1168,30 @@ private:
     */
     static bool checkColorValue(QString value, QColor &col);
 
-    bool m_fileChangedDialogsActivated;
+    bool m_fileChangedDialogsActivated = false;
 
     //
     // KTextEditor::ConfigInterface
     //
 public:
-    QStringList configKeys() const Q_DECL_OVERRIDE;
-    QVariant configValue(const QString &key) Q_DECL_OVERRIDE;
-    void setConfigValue(const QString &key, const QVariant &value) Q_DECL_OVERRIDE;
+    QStringList configKeys() const override;
+    QVariant configValue(const QString &key) override;
+    void setConfigValue(const QString &key, const QVariant &value) override;
 
     //
     // KTextEditor::RecoveryInterface
     //
 public:
-    bool isDataRecoveryAvailable() const Q_DECL_OVERRIDE;
-    void recoverData() Q_DECL_OVERRIDE;
-    void discardDataRecovery() Q_DECL_OVERRIDE;
+    bool isDataRecoveryAvailable() const override;
+    void recoverData() override;
+    void discardDataRecovery() override;
 
     //
     // Highlighting information
     //
 public:
-    QStringList embeddedHighlightingModes() const Q_DECL_OVERRIDE;
-    QString highlightingModeAt(const KTextEditor::Cursor &position) Q_DECL_OVERRIDE;
+    QStringList embeddedHighlightingModes() const override;
+    QString highlightingModeAt(const KTextEditor::Cursor &position) override;
     // TODO KDE5: move to View
     virtual KTextEditor::Attribute::Ptr attributeAt(const KTextEditor::Cursor &position);
 
@@ -1198,7 +1199,7 @@ public:
     //BEGIN: KTextEditor::MessageInterface
     //
 public:
-    bool postMessage(KTextEditor::Message *message) Q_DECL_OVERRIDE;
+    bool postMessage(KTextEditor::Message *message) override;
 
 public Q_SLOTS:
     void messageDestroyed(KTextEditor::Message *message);
@@ -1243,16 +1244,14 @@ public:
                              KTextEditor::DocumentPrivate::OffsetList &encToDecOffsetList);
     void replaceCharactersByEncoding(const KTextEditor::Range &range);
 
-    enum EncodedCharaterInsertionPolicy {EncodeAlways, EncodeWhenPresent, EncodeNever};
-
 protected:
-    KateOnTheFlyChecker *m_onTheFlyChecker;
+    KateOnTheFlyChecker *m_onTheFlyChecker = nullptr;
     QString m_defaultDictionary;
     QList<QPair<KTextEditor::MovingRange *, QString> > m_dictionaryRanges;
 
     // from KTextEditor::MovingRangeFeedback
-    void rangeInvalid(KTextEditor::MovingRange *movingRange) Q_DECL_OVERRIDE;
-    void rangeEmpty(KTextEditor::MovingRange *movingRange) Q_DECL_OVERRIDE;
+    void rangeInvalid(KTextEditor::MovingRange *movingRange) override;
+    void rangeEmpty(KTextEditor::MovingRange *movingRange) override;
 
     void deleteDictionaryRange(KTextEditor::MovingRange *movingRange);
 
@@ -1329,17 +1328,17 @@ private:
     /**
      * current state
      */
-    DocumentStates m_documentState;
+    DocumentStates m_documentState = DocumentIdle;
 
     /**
      * read-write state before loading started
      */
-    bool m_readWriteStateBeforeLoading;
+    bool m_readWriteStateBeforeLoading = false;
 
     /**
      * if the document is untitled
      */
-    bool m_isUntitled;
+    bool m_isUntitled = true;
     /**
      * loading job, we want to cancel with cancel in the loading message
      */
@@ -1353,7 +1352,7 @@ private:
     /**
      * Was there any open error on last file loading?
      */
-    bool m_openingError;
+    bool m_openingError = false;
 
     /**
      * Last open file error message
@@ -1362,7 +1361,7 @@ private:
 
 public:
     /**
-     * reads the line length limit from config, if it is not overriden
+     * reads the line length limit from config, if it is not overridden
      */
     int lineLengthLimit() const;
 
@@ -1387,7 +1386,7 @@ private:
      */
     QSharedPointer<KTextEditor::MovingRange> m_currentAutobraceRange;
     /**
-     * current autobrace closing charater (e.g. ']')
+     * current autobrace closing character (e.g. ']')
      */
     QChar m_currentAutobraceClosingChar;
 
@@ -1405,6 +1404,10 @@ private Q_SLOTS:
      * trigger a close of this document in the application
      */
     void closeDocumentInApplication();
+private:
+    // To calculate a QHash.keys() is quite expensive,
+    // better keep a copy of that list updated when a view is added or removed.
+    QList<KTextEditor::View *> m_viewsCache;
 };
 
 #endif
