@@ -53,9 +53,9 @@ KTextEditor::Range KatePlainTextSearch::search(const QString &text, const KTextE
     // abuse regex for whole word plaintext search
     if (m_wholeWords) {
         // escape dot and friends
-        const QString workPattern = QString::fromLatin1("\\b%1\\b").arg(QRegExp::escape(text));
+        const QString workPattern = QStringLiteral("\\b%1\\b").arg(QRegExp::escape(text));
 
-        return KateRegExpSearch(m_document, m_caseSensitivity).search(workPattern, inputRange, backwards)[0];
+        return KateRegExpSearch(m_document, m_caseSensitivity).search(workPattern, inputRange, backwards).at(0);
     }
 
     if (text.isEmpty() || !inputRange.isValid() || (inputRange.start() == inputRange.end())) {
@@ -63,7 +63,7 @@ KTextEditor::Range KatePlainTextSearch::search(const QString &text, const KTextE
     }
 
     // split multi-line needle into single lines
-    const QStringList needleLines = text.split(QLatin1String("\n"));
+    const QStringList needleLines = text.split(QStringLiteral("\n"));
 
     if (needleLines.count() > 1) {
         // multi-line plaintext search (both forwards or backwards)
@@ -86,14 +86,19 @@ KTextEditor::Range KatePlainTextSearch::search(const QString &text, const KTextE
                     if (forMin == j && startCol < inputRange.start().column()) {
                         break;
                     }
-                    if (!hayLine.endsWith(needleLine, m_caseSensitivity)) {
+
+                    // NOTE: QString("")::endsWith("") is false in Qt, therefore we need the additional checks.
+                    const bool endsWith = hayLine.endsWith(needleLine, m_caseSensitivity) || (hayLine.isEmpty() && needleLine.isEmpty());
+                    if (!endsWith) {
                         break;
                     }
                 } else if (k == needleLines.count() - 1) {
                     // last line
                     const int maxRight = (j + k == inputRange.end().line()) ? inputRange.end().column() : hayLine.length();
 
-                    if (hayLine.startsWith(needleLine, m_caseSensitivity) && needleLine.length() <= maxRight) {
+                    // NOTE: QString("")::startsWith("") is false in Qt, therefore we need the additional checks.
+                    const bool startsWith = hayLine.startsWith(needleLine, m_caseSensitivity) || (hayLine.isEmpty() && needleLine.isEmpty());
+                    if (startsWith && needleLine.length() <= maxRight) {
                         return KTextEditor::Range(j, startCol, j + k, needleLine.length());
                     }
                 } else {
