@@ -78,19 +78,19 @@ public:
         : m_insertBefore(QVector<QString>(expectedItemCount)),
           m_insertAfter(QVector<QString>(expectedItemCount)),
           m_indexWalker(0),
-          m_menu(NULL)
+          m_menu(nullptr)
     {
-        Q_ASSERT(parent != NULL);
+        Q_ASSERT(parent != nullptr);
         m_menu = parent->addMenu(i18n("Add..."));
-        if (m_menu == NULL) {
+        if (m_menu == nullptr) {
             return;
         }
-        m_menu->setIcon(QIcon::fromTheme(QLatin1String("list-add")));
+        m_menu->setIcon(QIcon::fromTheme(QStringLiteral("list-add")));
     }
 
     void enableMenu(bool enabled)
     {
-        if (m_menu == NULL) {
+        if (m_menu == nullptr) {
             return;
         }
         m_menu->setEnabled(enabled);
@@ -100,7 +100,7 @@ public:
                   const QString description, const QString &realBefore = QString(),
                   const QString &realAfter = QString())
     {
-        if (m_menu == NULL) {
+        if (m_menu == nullptr) {
             return;
         }
         QAction *const action = m_menu->addAction(before + after + QLatin1Char('\t') + description);
@@ -112,7 +112,7 @@ public:
 
     void addSeparator()
     {
-        if (m_menu == NULL) {
+        if (m_menu == nullptr) {
             return;
         }
         m_menu->addSeparator();
@@ -141,10 +141,10 @@ KateSearchBar::KateSearchBar(bool initAsPower, KTextEditor::ViewPrivate *view, K
       m_view(view),
       m_config(config),
       m_layout(new QVBoxLayout()),
-      m_widget(NULL),
-      m_incUi(NULL),
+      m_widget(nullptr),
+      m_incUi(nullptr),
       m_incInitCursor(view->cursorPosition()),
-      m_powerUi(NULL),
+      m_powerUi(nullptr),
       highlightMatchAttribute(new Attribute()),
       highlightReplacementAttribute(new Attribute()),
       m_incHighlightAll(false),
@@ -257,7 +257,7 @@ void KateSearchBar::findNext()
     const bool found = find();
 
     if (found) {
-        QComboBox *combo = m_powerUi != 0 ? m_powerUi->pattern : m_incUi->pattern;
+        QComboBox *combo = m_powerUi != nullptr ? m_powerUi->pattern : m_incUi->pattern;
 
         // Add to search history
         addCurrentTextToHistory(combo);
@@ -269,7 +269,7 @@ void KateSearchBar::findPrevious()
     const bool found = find(SearchBackward);
 
     if (found) {
-        QComboBox *combo = m_powerUi != 0 ? m_powerUi->pattern : m_incUi->pattern;
+        QComboBox *combo = m_powerUi != nullptr ? m_powerUi->pattern : m_incUi->pattern;
 
         // Add to search history
         addCurrentTextToHistory(combo);
@@ -337,7 +337,7 @@ void KateSearchBar::indicateMatch(MatchResult matchResult)
     }
 
     // Update status label
-    if (m_incUi != NULL) {
+    if (m_incUi != nullptr) {
         QPalette foreground(m_incUi->status->palette());
         switch (matchResult) {
         case MatchFound: // FALLTHROUGH
@@ -444,7 +444,7 @@ void KateSearchBar::onMatchCaseToggled(bool /*matchCase*/)
 {
     sendConfig();
 
-    if (m_incUi != 0) {
+    if (m_incUi != nullptr) {
         // Re-search with new settings
         const QString pattern = m_incUi->pattern->currentText();
         onIncPatternChanged(pattern);
@@ -572,7 +572,7 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString *replace
     match.searchText(inputRange, searchPattern());
     if (match.isValid() && match.range() == selection) {
         // Same match again
-        if (replacement != 0) {
+        if (replacement != nullptr) {
             // Selection is match -> replace
             KTextEditor::MovingRange *smartInputRange = m_view->doc()->newMovingRange(inputRange, KTextEditor::MovingRange::ExpandLeft | KTextEditor::MovingRange::ExpandRight);
             afterReplace = match.replace(*replacement, m_view->blockSelection());
@@ -583,10 +583,10 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString *replace
         if (!selectionOnly()) {
             // Find, second try after old selection
             if (searchDirection == SearchForward) {
-                const Cursor start = (replacement != 0) ? afterReplace.end() : selection.end();
+                const Cursor start = (replacement != nullptr) ? afterReplace.end() : selection.end();
                 inputRange.setRange(start, inputRange.end());
             } else {
-                const Cursor end = (replacement != 0) ? afterReplace.start() : selection.start();
+                const Cursor end = (replacement != nullptr) ? afterReplace.start() : selection.start();
                 inputRange.setRange(inputRange.start(), end);
             }
         }
@@ -608,28 +608,25 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString *replace
     if (askWrap) {
         QString question = searchDirection == SearchForward  ? i18n("Bottom of file reached. Continue from top?")
                            : i18n("Top of file reached. Continue from bottom?");
-        wrap = (KMessageBox::questionYesNo(0, question, i18n("Continue search?"), KStandardGuiItem::yes(), KStandardGuiItem::no(),
-                                           QLatin1String("DoNotShowAgainContinueSearchDialog")) == KMessageBox::Yes);
+        wrap = (KMessageBox::questionYesNo(nullptr, question, i18n("Continue search?"), KStandardGuiItem::yes(), KStandardGuiItem::no(),
+                                           QStringLiteral("DoNotShowAgainContinueSearchDialog")) == KMessageBox::Yes);
 
     }
     if (wrap) {
-        // show message widget when wrapping (if not already present)
-        if (searchDirection == SearchForward && !m_wrappedTopMessage) {
-            const QString msg = i18n("Continuing search from top");
-            m_wrappedTopMessage = new KTextEditor::Message(msg, KTextEditor::Message::Information);
-            m_wrappedTopMessage->setPosition(KTextEditor::Message::TopInView);
-            m_wrappedTopMessage->setAutoHide(2000);
-            m_wrappedTopMessage->setAutoHideMode(KTextEditor::Message::Immediate);
-            m_wrappedTopMessage->setView(m_view);
-            m_view->doc()->postMessage(m_wrappedTopMessage);
-        } else if (searchDirection == SearchBackward && !m_wrappedBottomMessage) {
-            const QString msg = i18n("Continuing search from bottom");
-            m_wrappedBottomMessage = new KTextEditor::Message(msg, KTextEditor::Message::Information);
-            m_wrappedBottomMessage->setPosition(KTextEditor::Message::BottomInView);
-            m_wrappedBottomMessage->setAutoHide(2000);
-            m_wrappedBottomMessage->setAutoHideMode(KTextEditor::Message::Immediate);
-            m_wrappedBottomMessage->setView(m_view);
-            m_view->doc()->postMessage(m_wrappedBottomMessage);
+        // show message widget when wrapping
+        const QIcon icon = searchDirection == SearchForward
+            ? QIcon::fromTheme(QStringLiteral("go-down-search"))
+            : QIcon::fromTheme(QStringLiteral("go-up-search"));
+
+        if (!m_wrappedMessage || m_lastSearchDirection != searchDirection) {
+            m_lastSearchDirection = searchDirection;
+            m_wrappedMessage = new KTextEditor::Message(i18n("Search wrapped"), KTextEditor::Message::Positive);
+            m_wrappedMessage->setIcon(icon);
+            m_wrappedMessage->setPosition(KTextEditor::Message::CenterInView);
+            m_wrappedMessage->setAutoHide(2000);
+            m_wrappedMessage->setAutoHideMode(KTextEditor::Message::Immediate);
+            m_wrappedMessage->setView(m_view);
+            m_view->doc()->postMessage(m_wrappedMessage);
         }
 
         inputRange = m_view->document()->documentRange();
@@ -665,7 +662,7 @@ void KateSearchBar::findAll()
     Range inputRange = (m_view->selection() && selectionOnly())
                        ? m_view->selectionRange()
                        : m_view->document()->documentRange();
-    const int occurrences = findAll(inputRange, NULL);
+    const int occurrences = findAll(inputRange, nullptr);
 
     // send passive notification to view
     showInfoMessage(i18ncp("short translation", "1 match found", "%1 matches found", occurrences));
@@ -732,7 +729,7 @@ void KateSearchBar::sendConfig()
     const long pastFlags = m_config->searchFlags();
     long futureFlags = pastFlags;
 
-    if (m_powerUi != NULL) {
+    if (m_powerUi != nullptr) {
         const bool OF_POWER = true;
         backupConfig(OF_POWER);
 
@@ -754,7 +751,7 @@ void KateSearchBar::sendConfig()
                                ? KateViewConfig::PowerModeWholeWords
                                : KateViewConfig::PowerModePlainText)));
 
-    } else if (m_incUi != NULL) {
+    } else if (m_incUi != nullptr) {
         const bool OF_INCREMENTAL = false;
         backupConfig(OF_INCREMENTAL);
 
@@ -826,7 +823,7 @@ int KateSearchBar::findAll(Range inputRange, const QString *replacement)
             bool const originalMatchEmpty = match.isEmpty();
 
             // Work with the match
-            if (replacement != NULL) {
+            if (replacement != nullptr) {
                 if (matchCounter == 0) {
                     static_cast<KTextEditor::DocumentPrivate *>(m_view->document())->startEditing();
                 }
@@ -870,17 +867,28 @@ int KateSearchBar::findAll(Range inputRange, const QString *replacement)
 
     // After last match
     if (matchCounter > 0) {
-        if (replacement != NULL) {
+        if (replacement != nullptr) {
             static_cast<KTextEditor::DocumentPrivate *>(m_view->document())->finishEditing();
         }
     }
 
-    if (replacement == NULL)
-        foreach (const Range &r, highlightRanges) {
+    // Add ScrollBarMarks
+    KTextEditor::MarkInterface* iface = qobject_cast<KTextEditor::MarkInterface*>(m_view->document());
+    if (iface) {
+        iface->setMarkDescription(KTextEditor::MarkInterface::SearchMatch, i18n("SearchHighLight"));
+        iface->setMarkPixmap(KTextEditor::MarkInterface::SearchMatch, QIcon().pixmap(0,0));
+        foreach (Range r, highlightRanges) {
+            iface->addMark(r.start().line(), KTextEditor::MarkInterface::SearchMatch);
+        }
+    }
+
+    // Add highlights
+    if (replacement == nullptr)
+        foreach (Range r, highlightRanges) {
             highlightMatch(r);
         }
     else
-        foreach (const Range &r, highlightRanges) {
+        foreach (Range r, highlightRanges) {
             highlightReplacement(r);
         }
 
@@ -938,7 +946,7 @@ void KateSearchBar::setSearchPattern(const QString &searchPattern)
 
 QString KateSearchBar::searchPattern() const
 {
-    return (m_powerUi != 0) ? m_powerUi->pattern->currentText()
+    return (m_powerUi != nullptr) ? m_powerUi->pattern->currentText()
            : m_incUi->pattern->currentText();
 }
 
@@ -971,7 +979,7 @@ KTextEditor::SearchOptions KateSearchBar::searchOptions(SearchDirection searchDi
         enabledOptions |= Backwards;
     }
 
-    if (m_powerUi != NULL) {
+    if (m_powerUi != nullptr) {
         switch (m_powerUi->searchMode->currentIndex()) {
         case MODE_WHOLE_WORDS:
             enabledOptions |= WholeWords;
@@ -1078,7 +1086,7 @@ void KateSearchBar::showExtendedContextMenu(bool forPattern, const QPoint &pos)
     QComboBox *comboBox = forPattern ? m_powerUi->pattern : m_powerUi->replacement;
     QMenu *const contextMenu = comboBox->lineEdit()->createStandardContextMenu();
 
-    if (contextMenu == NULL) {
+    if (contextMenu == nullptr) {
         return;
     }
 
@@ -1104,24 +1112,24 @@ void KateSearchBar::showExtendedContextMenu(bool forPattern, const QPoint &pos)
         // Build menu
         if (forPattern) {
             if (regexMode) {
-                addMenuManager.addEntry(QLatin1String("^"), QString(), i18n("Beginning of line"));
-                addMenuManager.addEntry(QLatin1String("$"), QString(), i18n("End of line"));
+                addMenuManager.addEntry(QStringLiteral("^"), QString(), i18n("Beginning of line"));
+                addMenuManager.addEntry(QStringLiteral("$"), QString(), i18n("End of line"));
                 addMenuManager.addSeparator();
-                addMenuManager.addEntry(QLatin1String("."), QString(), i18n("Any single character (excluding line breaks)"));
+                addMenuManager.addEntry(QStringLiteral("."), QString(), i18n("Any single character (excluding line breaks)"));
                 addMenuManager.addSeparator();
-                addMenuManager.addEntry(QLatin1String("+"), QString(), i18n("One or more occurrences"));
-                addMenuManager.addEntry(QLatin1String("*"), QString(), i18n("Zero or more occurrences"));
-                addMenuManager.addEntry(QLatin1String("?"), QString(), i18n("Zero or one occurrences"));
-                addMenuManager.addEntry(QLatin1String("{a"), QLatin1String(",b}"), i18n("<a> through <b> occurrences"), QLatin1String("{"), QLatin1String(",}"));
+                addMenuManager.addEntry(QStringLiteral("+"), QString(), i18n("One or more occurrences"));
+                addMenuManager.addEntry(QStringLiteral("*"), QString(), i18n("Zero or more occurrences"));
+                addMenuManager.addEntry(QStringLiteral("?"), QString(), i18n("Zero or one occurrences"));
+                addMenuManager.addEntry(QStringLiteral("{a"), QStringLiteral(",b}"), i18n("<a> through <b> occurrences"), QStringLiteral("{"), QStringLiteral(",}"));
                 addMenuManager.addSeparator();
-                addMenuManager.addEntry(QLatin1String("("), QLatin1String(")"), i18n("Group, capturing"));
-                addMenuManager.addEntry(QLatin1String("|"), QString(), i18n("Or"));
-                addMenuManager.addEntry(QLatin1String("["), QLatin1String("]"), i18n("Set of characters"));
-                addMenuManager.addEntry(QLatin1String("[^"), QLatin1String("]"), i18n("Negative set of characters"));
+                addMenuManager.addEntry(QStringLiteral("("), QStringLiteral(")"), i18n("Group, capturing"));
+                addMenuManager.addEntry(QStringLiteral("|"), QString(), i18n("Or"));
+                addMenuManager.addEntry(QStringLiteral("["), QStringLiteral("]"), i18n("Set of characters"));
+                addMenuManager.addEntry(QStringLiteral("[^"), QStringLiteral("]"), i18n("Negative set of characters"));
                 addMenuManager.addSeparator();
             }
         } else {
-            addMenuManager.addEntry(QLatin1String("\\0"), QString(), i18n("Whole match reference"));
+            addMenuManager.addEntry(QStringLiteral("\\0"), QString(), i18n("Whole match reference"));
             addMenuManager.addSeparator();
             if (regexMode) {
                 const QString pattern = m_powerUi->pattern->currentText();
@@ -1141,45 +1149,45 @@ void KateSearchBar::showExtendedContextMenu(bool forPattern, const QPoint &pos)
             }
         }
 
-        addMenuManager.addEntry(QLatin1String("\\n"), QString(), i18n("Line break"));
-        addMenuManager.addEntry(QLatin1String("\\t"), QString(), i18n("Tab"));
+        addMenuManager.addEntry(QStringLiteral("\\n"), QString(), i18n("Line break"));
+        addMenuManager.addEntry(QStringLiteral("\\t"), QString(), i18n("Tab"));
 
         if (forPattern && regexMode) {
-            addMenuManager.addEntry(QLatin1String("\\b"), QString(), i18n("Word boundary"));
-            addMenuManager.addEntry(QLatin1String("\\B"), QString(), i18n("Not word boundary"));
-            addMenuManager.addEntry(QLatin1String("\\d"), QString(), i18n("Digit"));
-            addMenuManager.addEntry(QLatin1String("\\D"), QString(), i18n("Non-digit"));
-            addMenuManager.addEntry(QLatin1String("\\s"), QString(), i18n("Whitespace (excluding line breaks)"));
-            addMenuManager.addEntry(QLatin1String("\\S"), QString(), i18n("Non-whitespace (excluding line breaks)"));
-            addMenuManager.addEntry(QLatin1String("\\w"), QString(), i18n("Word character (alphanumerics plus '_')"));
-            addMenuManager.addEntry(QLatin1String("\\W"), QString(), i18n("Non-word character"));
+            addMenuManager.addEntry(QStringLiteral("\\b"), QString(), i18n("Word boundary"));
+            addMenuManager.addEntry(QStringLiteral("\\B"), QString(), i18n("Not word boundary"));
+            addMenuManager.addEntry(QStringLiteral("\\d"), QString(), i18n("Digit"));
+            addMenuManager.addEntry(QStringLiteral("\\D"), QString(), i18n("Non-digit"));
+            addMenuManager.addEntry(QStringLiteral("\\s"), QString(), i18n("Whitespace (excluding line breaks)"));
+            addMenuManager.addEntry(QStringLiteral("\\S"), QString(), i18n("Non-whitespace (excluding line breaks)"));
+            addMenuManager.addEntry(QStringLiteral("\\w"), QString(), i18n("Word character (alphanumerics plus '_')"));
+            addMenuManager.addEntry(QStringLiteral("\\W"), QString(), i18n("Non-word character"));
         }
 
-        addMenuManager.addEntry(QLatin1String("\\0???"), QString(), i18n("Octal character 000 to 377 (2^8-1)"), QLatin1String("\\0"));
-        addMenuManager.addEntry(QLatin1String("\\x????"), QString(), i18n("Hex character 0000 to FFFF (2^16-1)"), QLatin1String("\\x"));
-        addMenuManager.addEntry(QLatin1String("\\\\"), QString(), i18n("Backslash"));
+        addMenuManager.addEntry(QStringLiteral("\\0???"), QString(), i18n("Octal character 000 to 377 (2^8-1)"), QStringLiteral("\\0"));
+        addMenuManager.addEntry(QStringLiteral("\\x????"), QString(), i18n("Hex character 0000 to FFFF (2^16-1)"), QStringLiteral("\\x"));
+        addMenuManager.addEntry(QStringLiteral("\\\\"), QString(), i18n("Backslash"));
 
         if (forPattern && regexMode) {
             addMenuManager.addSeparator();
-            addMenuManager.addEntry(QLatin1String("(?:E"), QLatin1String(")"), i18n("Group, non-capturing"), QLatin1String("(?:"));
-            addMenuManager.addEntry(QLatin1String("(?=E"), QLatin1String(")"), i18n("Lookahead"), QLatin1String("(?="));
-            addMenuManager.addEntry(QLatin1String("(?!E"), QLatin1String(")"), i18n("Negative lookahead"), QLatin1String("(?!"));
+            addMenuManager.addEntry(QStringLiteral("(?:E"), QStringLiteral(")"), i18n("Group, non-capturing"), QStringLiteral("(?:"));
+            addMenuManager.addEntry(QStringLiteral("(?=E"), QStringLiteral(")"), i18n("Lookahead"), QStringLiteral("(?="));
+            addMenuManager.addEntry(QStringLiteral("(?!E"), QStringLiteral(")"), i18n("Negative lookahead"), QStringLiteral("(?!"));
         }
 
         if (!forPattern) {
             addMenuManager.addSeparator();
-            addMenuManager.addEntry(QLatin1String("\\L"), QString(), i18n("Begin lowercase conversion"));
-            addMenuManager.addEntry(QLatin1String("\\U"), QString(), i18n("Begin uppercase conversion"));
-            addMenuManager.addEntry(QLatin1String("\\E"), QString(), i18n("End case conversion"));
-            addMenuManager.addEntry(QLatin1String("\\l"), QString(), i18n("Lowercase first character conversion"));
-            addMenuManager.addEntry(QLatin1String("\\u"), QString(), i18n("Uppercase first character conversion"));
-            addMenuManager.addEntry(QLatin1String("\\#[#..]"), QString(), i18n("Replacement counter (for Replace All)"), QLatin1String("\\#"));
+            addMenuManager.addEntry(QStringLiteral("\\L"), QString(), i18n("Begin lowercase conversion"));
+            addMenuManager.addEntry(QStringLiteral("\\U"), QString(), i18n("Begin uppercase conversion"));
+            addMenuManager.addEntry(QStringLiteral("\\E"), QString(), i18n("End case conversion"));
+            addMenuManager.addEntry(QStringLiteral("\\l"), QString(), i18n("Lowercase first character conversion"));
+            addMenuManager.addEntry(QStringLiteral("\\u"), QString(), i18n("Uppercase first character conversion"));
+            addMenuManager.addEntry(QStringLiteral("\\#[#..]"), QString(), i18n("Replacement counter (for Replace All)"), QStringLiteral("\\#"));
         }
     }
 
     // Show menu
     QAction *const result = contextMenu->exec(comboBox->mapToGlobal(pos));
-    if (result != NULL) {
+    if (result != nullptr) {
         addMenuManager.handle(result, comboBox->lineEdit());
     }
 }
@@ -1267,34 +1275,34 @@ void KateSearchBar::enterPowerMode()
     // If there's no new selection, we'll use the existing pattern
     if (initialPattern.isNull()) {
         // Coming from power search?
-        const bool fromReplace = (m_powerUi != NULL) && (m_widget->isVisible());
+        const bool fromReplace = (m_powerUi != nullptr) && (m_widget->isVisible());
         if (fromReplace) {
             QLineEdit *const patternLineEdit = m_powerUi->pattern->lineEdit();
-            Q_ASSERT(patternLineEdit != NULL);
+            Q_ASSERT(patternLineEdit != nullptr);
             patternLineEdit->selectAll();
             m_powerUi->pattern->setFocus(Qt::MouseFocusReason);
             return;
         }
 
         // Coming from incremental search?
-        const bool fromIncremental = (m_incUi != NULL) && (m_widget->isVisible());
+        const bool fromIncremental = (m_incUi != nullptr) && (m_widget->isVisible());
         if (fromIncremental) {
             initialPattern = m_incUi->pattern->currentText();
         }
     }
 
     // Create dialog
-    const bool create = (m_powerUi == NULL);
+    const bool create = (m_powerUi == nullptr);
     if (create) {
         // Kill incremental widget
-        if (m_incUi != NULL) {
+        if (m_incUi != nullptr) {
             // Backup current settings
             const bool OF_INCREMENTAL = false;
             backupConfig(OF_INCREMENTAL);
 
             // Kill widget
             delete m_incUi;
-            m_incUi = NULL;
+            m_incUi = nullptr;
             m_layout->removeWidget(m_widget);
             m_widget->deleteLater(); // I didn't get a crash here but for symmetrie to the other mutate slot^
         }
@@ -1311,22 +1319,25 @@ void KateSearchBar::enterPowerMode()
         m_powerUi->pattern->setMaxCount(m_config->maxHistorySize());
         m_powerUi->pattern->setModel(KTextEditor::EditorPrivate::self()->searchHistoryModel());
         m_powerUi->pattern->lineEdit()->setClearButtonEnabled(true);
-        m_powerUi->pattern->setCompleter(0);
+        m_powerUi->pattern->setCompleter(nullptr);
         m_powerUi->replacement->setDuplicatesEnabled(false);
         m_powerUi->replacement->setInsertPolicy(QComboBox::InsertAtTop);
         m_powerUi->replacement->setMaxCount(m_config->maxHistorySize());
         m_powerUi->replacement->setModel(KTextEditor::EditorPrivate::self()->replaceHistoryModel());
         m_powerUi->replacement->lineEdit()->setClearButtonEnabled(true);
-        m_powerUi->replacement->setCompleter(0);
+        m_powerUi->replacement->setCompleter(nullptr);
 
         // Icons
-        m_powerUi->mutate->setIcon(QIcon::fromTheme(QLatin1String("games-config-options")));
+        // Gnome does not seem to have all icons we want, so we use fall-back icons for those that are missing.
+        QIcon mutateIcon = QIcon::fromTheme(QStringLiteral("games-config-options"), QIcon::fromTheme(QStringLiteral("preferences-system")));
+        QIcon matchCaseIcon = QIcon::fromTheme(QStringLiteral("format-text-superscript"), QIcon::fromTheme(QStringLiteral("format-text-bold")));
+        m_powerUi->mutate->setIcon(mutateIcon);
         m_powerUi->mutate->setChecked(true);
-        m_powerUi->findNext->setIcon(QIcon::fromTheme(QLatin1String("go-down-search")));
-        m_powerUi->findPrev->setIcon(QIcon::fromTheme(QLatin1String("go-up-search")));
-        m_powerUi->findAll->setIcon(QIcon::fromTheme(QLatin1String("edit-find")));
-        m_powerUi->matchCase->setIcon(QIcon::fromTheme(QLatin1String("format-text-superscript")));
-        m_powerUi->selectionOnly->setIcon(QIcon::fromTheme(QLatin1String("edit-select-all")));
+        m_powerUi->findNext->setIcon(QIcon::fromTheme(QStringLiteral("go-down-search")));
+        m_powerUi->findPrev->setIcon(QIcon::fromTheme(QStringLiteral("go-up-search")));
+        m_powerUi->findAll->setIcon(QIcon::fromTheme(QStringLiteral("edit-find")));
+        m_powerUi->matchCase->setIcon(matchCaseIcon);
+        m_powerUi->selectionOnly->setIcon(QIcon::fromTheme(QStringLiteral("edit-select-all")));
 
         // Focus proxy
         centralWidget()->setFocusProxy(m_powerUi->pattern);
@@ -1346,13 +1357,13 @@ void KateSearchBar::enterPowerMode()
 
     // Set initial search pattern
     QLineEdit *const patternLineEdit = m_powerUi->pattern->lineEdit();
-    Q_ASSERT(patternLineEdit != NULL);
+    Q_ASSERT(patternLineEdit != nullptr);
     patternLineEdit->setText(initialPattern);
     patternLineEdit->selectAll();
 
     // Set initial replacement text
     QLineEdit *const replacementLineEdit = m_powerUi->replacement->lineEdit();
-    Q_ASSERT(replacementLineEdit != NULL);
+    Q_ASSERT(replacementLineEdit != nullptr);
     replacementLineEdit->setText(QString());
 
     // Propagate settings (slots are still inactive on purpose)
@@ -1407,7 +1418,7 @@ void KateSearchBar::enterIncrementalMode()
     // If there's no new selection, we'll use the existing pattern
     if (initialPattern.isNull()) {
         // Coming from incremental search?
-        const bool fromIncremental = (m_incUi != NULL) && (m_widget->isVisible());
+        const bool fromIncremental = (m_incUi != nullptr) && (m_widget->isVisible());
         if (fromIncremental) {
             m_incUi->pattern->lineEdit()->selectAll();
             m_incUi->pattern->setFocus(Qt::MouseFocusReason);
@@ -1415,7 +1426,7 @@ void KateSearchBar::enterIncrementalMode()
         }
 
         // Coming from power search?
-        const bool fromReplace = (m_powerUi != NULL) && (m_widget->isVisible());
+        const bool fromReplace = (m_powerUi != nullptr) && (m_widget->isVisible());
         if (fromReplace) {
             initialPattern = m_powerUi->pattern->currentText();
         }
@@ -1428,17 +1439,17 @@ void KateSearchBar::enterIncrementalMode()
     }
 
     // Create dialog
-    const bool create = (m_incUi == NULL);
+    const bool create = (m_incUi == nullptr);
     if (create) {
         // Kill power widget
-        if (m_powerUi != NULL) {
+        if (m_powerUi != nullptr) {
             // Backup current settings
             const bool OF_POWER = true;
             backupConfig(OF_POWER);
 
             // Kill widget
             delete m_powerUi;
-            m_powerUi = NULL;
+            m_powerUi = nullptr;
             m_layout->removeWidget(m_widget);
             m_widget->deleteLater(); //deleteLater, because it's not a good idea too delete the widget and there for the button triggering this slot
         }
@@ -1454,10 +1465,13 @@ void KateSearchBar::enterIncrementalMode()
 //             new QShortcut(KStandardShortcut::paste().alternate(), m_incUi->pattern, SLOT(paste()), 0, Qt::WidgetWithChildrenShortcut);
 
         // Icons
-        m_incUi->mutate->setIcon(QIcon::fromTheme(QLatin1String("games-config-options")));
-        m_incUi->next->setIcon(QIcon::fromTheme(QLatin1String("go-down-search")));
-        m_incUi->prev->setIcon(QIcon::fromTheme(QLatin1String("go-up-search")));
-        m_incUi->matchCase->setIcon(QIcon::fromTheme(QLatin1String("format-text-superscript")));
+        // Gnome does not seem to have all icons we want, so we use fall-back icons for those that are missing.
+        QIcon mutateIcon = QIcon::fromTheme(QStringLiteral("games-config-options"), QIcon::fromTheme(QStringLiteral("preferences-system")));
+        QIcon matchCaseIcon = QIcon::fromTheme(QStringLiteral("format-text-superscript"), QIcon::fromTheme(QStringLiteral("format-text-bold")));
+        m_incUi->mutate->setIcon(mutateIcon);
+        m_incUi->next->setIcon(QIcon::fromTheme(QStringLiteral("go-down-search")));
+        m_incUi->prev->setIcon(QIcon::fromTheme(QStringLiteral("go-up-search")));
+        m_incUi->matchCase->setIcon(matchCaseIcon);
 
         // Ensure minimum size
         m_incUi->pattern->setMinimumWidth(12 * m_incUi->pattern->fontMetrics().height());
@@ -1473,7 +1487,7 @@ void KateSearchBar::enterIncrementalMode()
         m_incUi->pattern->setMaxCount(m_config->maxHistorySize());
         m_incUi->pattern->setModel(KTextEditor::EditorPrivate::self()->searchHistoryModel());
         m_incUi->pattern->lineEdit()->setClearButtonEnabled(true);
-        m_incUi->pattern->setCompleter(0);
+        m_incUi->pattern->setCompleter(nullptr);
     }
 
     // Restore previous settings
@@ -1519,6 +1533,19 @@ void KateSearchBar::enterIncrementalMode()
 
 bool KateSearchBar::clearHighlights()
 {
+    // Remove ScrollBarMarks
+    KTextEditor::MarkInterface* iface = qobject_cast<KTextEditor::MarkInterface*>(m_view->document());
+    if (iface) {
+        const QHash<int, KTextEditor::Mark*> marks = iface->marks();
+        QHashIterator<int, KTextEditor::Mark*> i(marks);
+        while (i.hasNext()) {
+            i.next();
+            if (i.value()->type & KTextEditor::MarkInterface::SearchMatch) {
+                iface->removeMark(i.value()->line, KTextEditor::MarkInterface::SearchMatch);
+            }
+        }
+    }
+
     if (m_infoMessage) {
         delete m_infoMessage;
     }
@@ -1533,22 +1560,27 @@ bool KateSearchBar::clearHighlights()
 
 void KateSearchBar::updateHighlightColors()
 {
+    const QColor foregroundColor = m_view->defaultStyleAttribute(KTextEditor::dsNormal)->foreground().color();
     const QColor &searchColor = m_view->renderer()->config()->searchHighlightColor();
     const QColor &replaceColor = m_view->renderer()->config()->replaceHighlightColor();
 
     // init match attribute
+    highlightMatchAttribute->setForeground(foregroundColor);
     highlightMatchAttribute->setBackground(searchColor);
     highlightMatchAttribute->dynamicAttribute(Attribute::ActivateMouseIn)->setBackground(searchColor);
+    highlightMatchAttribute->dynamicAttribute(Attribute::ActivateMouseIn)->setForeground(foregroundColor);
     highlightMatchAttribute->dynamicAttribute(Attribute::ActivateCaretIn)->setBackground(searchColor);
+    highlightMatchAttribute->dynamicAttribute(Attribute::ActivateCaretIn)->setForeground(foregroundColor);
 
     // init replacement attribute
     highlightReplacementAttribute->setBackground(replaceColor);
+    highlightReplacementAttribute->setForeground(foregroundColor);
 }
 
 void KateSearchBar::showEvent(QShowEvent *event)
 {
     // Update init cursor
-    if (m_incUi != NULL) {
+    if (m_incUi != nullptr) {
         m_incInitCursor = m_view->cursorPosition();
     }
 
@@ -1558,7 +1590,7 @@ void KateSearchBar::showEvent(QShowEvent *event)
 
 void KateSearchBar::updateSelectionOnly()
 {
-    if (m_powerUi == NULL) {
+    if (m_powerUi == nullptr) {
         return;
     }
 
@@ -1574,7 +1606,7 @@ void KateSearchBar::updateSelectionOnly()
 
 void KateSearchBar::updateIncInitCursor()
 {
-    if (m_incUi == NULL) {
+    if (m_incUi == nullptr) {
         return;
     }
 
@@ -1606,7 +1638,7 @@ void KateSearchBar::onPowerReplacmentContextMenuRequest()
 
 bool KateSearchBar::isPower() const
 {
-    return m_powerUi != 0;
+    return m_powerUi != nullptr;
 }
 
 void KateSearchBar::slotReadWriteChanged()
